/*!
 * Copyright (c) Microsoft Corporation and contributors. All rights reserved.
 * Licensed under the MIT License.
 */
import { type IForestSubscription, type MapTree, type TreeValue } from "../core/index.js";
import { type FlexTreeField } from "../feature-libraries/index.js";
import { type TreeNode } from "./core/index.js";
/**
 * Retrieve the associated {@link TreeNode} for the given field's content.
 */
export declare function getTreeNodeForField(field: FlexTreeField): TreeNode | TreeValue | undefined;
/**
 * Records any proxies in the given content tree and does the necessary bookkeeping to ensure they are synchronized with subsequent reads of the tree.
 * @remarks If the content tree contains any proxies, this function must be called just prior to inserting the content into the tree.
 * Specifically, no other content may be inserted into the tree between the invocation of this function and the insertion of `content`.
 * The insertion of `content` must occur or else this function will cause memory leaks.
 * @param content - the tree of content to be inserted, of which any of its object/map/array nodes might be a proxy
 * @param anchors - the {@link AnchorSet} for the tree
 * @returns The content after having all proxies replaced inline with plain javascript objects.
 * See {@link extractFactoryContent} for more details.
 */
export declare function prepareContentForHydration(content: MapTree | readonly MapTree[] | undefined, forest: IForestSubscription): void;
//# sourceMappingURL=proxies.d.ts.map