/*!
 * Copyright (c) Microsoft Corporation and contributors. All rights reserved.
 * Licensed under the MIT License.
 */
import type { StableId } from "@fluidframework/id-compressor";
import type { LocalNodeKey, StableNodeKey } from "./nodeKey.js";
import { type NodeKeyManager } from "./nodeKeyManager.js";
/**
 * Mock {@link NodeKeyManager} that generates deterministic {@link StableNodeKey}s and {@link LocalNodeKey}s.
 * @remarks This is useful for test environments because it will always yield the same keys in the same order.
 * It should not be used for production environments for the same reason; the {@link StableNodeKey}s are not universally unique.
 */
export declare class MockNodeKeyManager implements NodeKeyManager {
    private count;
    generateStableNodeKey(): StableNodeKey;
    generateLocalNodeKey(): LocalNodeKey;
    localizeNodeKey(key: StableNodeKey): LocalNodeKey;
    stabilizeNodeKey(key: LocalNodeKey): StableNodeKey;
    tryLocalizeNodeKey(key: string): LocalNodeKey | undefined;
    getId(offset: number): StableId;
}
//# sourceMappingURL=mockNodeKeyManager.d.ts.map