/*!
 * Copyright (c) Microsoft Corporation and contributors. All rights reserved.
 * Licensed under the MIT License.
 */
var __runInitializers = (this && this.__runInitializers) || function (thisArg, initializers, value) {
    var useValue = arguments.length > 2;
    for (var i = 0; i < initializers.length; i++) {
        value = useValue ? initializers[i].call(thisArg, value) : initializers[i].call(thisArg);
    }
    return useValue ? value : void 0;
};
var __esDecorate = (this && this.__esDecorate) || function (ctor, descriptorIn, decorators, contextIn, initializers, extraInitializers) {
    function accept(f) { if (f !== void 0 && typeof f !== "function") throw new TypeError("Function expected"); return f; }
    var kind = contextIn.kind, key = kind === "getter" ? "get" : kind === "setter" ? "set" : "value";
    var target = !descriptorIn && ctor ? contextIn["static"] ? ctor : ctor.prototype : null;
    var descriptor = descriptorIn || (target ? Object.getOwnPropertyDescriptor(target, contextIn.name) : {});
    var _, done = false;
    for (var i = decorators.length - 1; i >= 0; i--) {
        var context = {};
        for (var p in contextIn) context[p] = p === "access" ? {} : contextIn[p];
        for (var p in contextIn.access) context.access[p] = contextIn.access[p];
        context.addInitializer = function (f) { if (done) throw new TypeError("Cannot add initializers after decoration has completed"); extraInitializers.push(accept(f || null)); };
        var result = (0, decorators[i])(kind === "accessor" ? { get: descriptor.get, set: descriptor.set } : descriptor[key], context);
        if (kind === "accessor") {
            if (result === void 0) continue;
            if (result === null || typeof result !== "object") throw new TypeError("Object expected");
            if (_ = accept(result.get)) descriptor.get = _;
            if (_ = accept(result.set)) descriptor.set = _;
            if (_ = accept(result.init)) initializers.unshift(_);
        }
        else if (_ = accept(result)) {
            if (kind === "field") initializers.unshift(_);
            else descriptor[key] = _;
        }
    }
    if (target) Object.defineProperty(target, contextIn.name, descriptor);
    done = true;
};
var __setFunctionName = (this && this.__setFunctionName) || function (f, name, prefix) {
    if (typeof name === "symbol") name = name.description ? "[".concat(name.description, "]") : "";
    return Object.defineProperty(f, "name", { configurable: true, value: prefix ? "".concat(prefix, " ", name) : name });
};
import { assert } from "@fluidframework/core-utils/internal";
import { SummaryTreeBuilder } from "@fluidframework/runtime-utils/internal";
import { SharedObject, } from "@fluidframework/shared-object-base/internal";
import { findAncestor, RevisionTagCodec, } from "../core/index.js";
import { brand, Breakable, throwIfBroken, breakingClass, } from "../util/index.js";
import { EditManager, minimumPossibleSequenceNumber } from "./editManager.js";
import { makeEditManagerCodec } from "./editManagerCodecs.js";
import { EditManagerSummarizer } from "./editManagerSummarizer.js";
import { makeMessageCodec } from "./messageCodecs.js";
import { NoOpChangeEnricher } from "./changeEnricher.js";
import { DefaultResubmitMachine } from "./defaultResubmitMachine.js";
import { BranchCommitEnricher } from "./branchCommitEnricher.js";
import { createChildLogger } from "@fluidframework/telemetry-utils/internal";
// TODO: Organize this to be adjacent to persisted types.
const summarizablesTreeKey = "indexes";
/**
 * Generic shared tree, which needs to be configured with indexes, field kinds and other configuration.
 */
let SharedTreeCore = (() => {
    let _classDecorators = [breakingClass];
    let _classDescriptor;
    let _classExtraInitializers = [];
    let _classThis;
    let _classSuper = SharedObject;
    let _instanceExtraInitializers = [];
    let _summarizeCore_decorators;
    var SharedTreeCore = _classThis = class extends _classSuper {
        /**
         * Used to edit the state of the tree. Edits will be immediately applied locally to the tree.
         * If there is no transaction currently ongoing, then the edits will be submitted to Fluid immediately as well.
         */
        get editor() {
            return this.getLocalBranch().editor;
        }
        /**
         * @param summarizables - Summarizers for all indexes used by this tree
         * @param changeFamily - The change family
         * @param editManager - The edit manager
         * @param id - The id of the shared object
         * @param runtime - The IFluidDataStoreRuntime which contains the shared object
         * @param attributes - Attributes of the shared object
         * @param telemetryContextPrefix - The property prefix for telemetry pertaining to this object. See {@link ITelemetryContext}
         */
        constructor(summarizables, changeFamily, options, formatOptions, 
        // Base class arguments
        id, runtime, attributes, telemetryContextPrefix, schema, schemaPolicy, resubmitMachine, enricher) {
            super(id, runtime, attributes, telemetryContextPrefix);
            this.breaker = (__runInitializers(this, _instanceExtraInitializers), new Breakable("Shared Tree"));
            /**
             * The sequence number that this instance is at.
             * This number is artificial in that it is made up by this instance as opposed to being provided by the runtime.
             * Is `undefined` after (and only after) this instance is attached.
             */
            this.detachedRevision = minimumPossibleSequenceNumber;
            this.schemaAndPolicy = {
                schema,
                policy: schemaPolicy,
            };
            const rebaseLogger = createChildLogger({
                logger: this.logger,
                namespace: "Rebase",
            });
            assert(runtime.idCompressor !== undefined, 0x886 /* IdCompressor must be enabled to use SharedTree */);
            this.idCompressor = runtime.idCompressor;
            this.mintRevisionTag = () => this.idCompressor.generateCompressedId();
            /**
             * A random ID that uniquely identifies this client in the collab session.
             * This is sent alongside every op to identify which client the op originated from.
             * This is used rather than the Fluid client ID because the Fluid client ID is not stable across reconnections.
             */
            const localSessionId = runtime.idCompressor.localSessionId;
            this.editManager = new EditManager(changeFamily, localSessionId, this.mintRevisionTag, rebaseLogger);
            this.editManager.localBranch.events.on("beforeChange", (change) => {
                if (this.detachedRevision === undefined) {
                    // Commit enrichment is only necessary for changes that will be submitted as ops, and changes issued while detached are not submitted.
                    this.commitEnricher.processChange(change);
                }
                if (change.type === "append") {
                    for (const commit of change.newCommits) {
                        this.submitCommit(commit, this.schemaAndPolicy, false);
                    }
                }
            });
            const revisionTagCodec = new RevisionTagCodec(runtime.idCompressor);
            const editManagerCodec = makeEditManagerCodec(this.editManager.changeFamily.codecs, revisionTagCodec, options, formatOptions.editManager);
            this.summarizables = [
                new EditManagerSummarizer(this.editManager, editManagerCodec, this.idCompressor, this.schemaAndPolicy),
                ...summarizables,
            ];
            assert(new Set(this.summarizables.map((e) => e.key)).size === this.summarizables.length, 0x350 /* Index summary element keys must be unique */);
            this.messageCodec = makeMessageCodec(changeFamily.codecs, new RevisionTagCodec(runtime.idCompressor), options, formatOptions.message);
            const changeEnricher = enricher ?? new NoOpChangeEnricher();
            this.resubmitMachine =
                resubmitMachine ??
                    new DefaultResubmitMachine((change) => changeFamily.rebaser.invert(change, true, this.mintRevisionTag()), changeEnricher);
            this.commitEnricher = new BranchCommitEnricher(changeFamily.rebaser, changeEnricher);
        }
        // TODO: SharedObject's merging of the two summary methods into summarizeCore is not what we want here:
        // We might want to not subclass it, or override/reimplement most of its functionality.
        summarizeCore(serializer, telemetryContext, incrementalSummaryContext) {
            const builder = new SummaryTreeBuilder();
            const summarizableBuilder = new SummaryTreeBuilder();
            // Merge the summaries of all summarizables together under a single ISummaryTree
            for (const s of this.summarizables) {
                summarizableBuilder.addWithStats(s.key, s.getAttachSummary((contents) => serializer.stringify(contents, this.handle), undefined, undefined, telemetryContext, incrementalSummaryContext));
            }
            builder.addWithStats(summarizablesTreeKey, summarizableBuilder.getSummaryTree());
            return builder.getSummaryTree();
        }
        async loadCore(services) {
            assert(this.editManager.localBranch.getHead() === this.editManager.getTrunkHead(), 0xaaa /* All local changes should be applied to the trunk before loading from summary */);
            const [editManagerSummarizer, ...summarizables] = this.summarizables;
            const loadEditManager = this.loadSummarizable(editManagerSummarizer, services);
            const loadSummarizables = summarizables.map(async (s) => this.loadSummarizable(s, services));
            if (this.detachedRevision !== undefined) {
                // If we are detached but loading from a summary, then we need to update our detached revision to ensure that it is ahead of all detached revisions in the summary.
                // First, finish loading the edit manager so that we can inspect the sequence numbers of the commits on the trunk.
                await loadEditManager;
                // Find the most recent detached revision in the summary trunk...
                let latestDetachedSequenceNumber;
                findAncestor(this.editManager.getTrunkHead(), (c) => {
                    const sequenceNumber = this.editManager.getSequenceNumber(c);
                    if (sequenceNumber !== undefined && sequenceNumber < 0) {
                        latestDetachedSequenceNumber = sequenceNumber;
                        return true;
                    }
                    return false;
                });
                // ...and set our detached revision to be as it would be if we had been already created that revision.
                this.detachedRevision = latestDetachedSequenceNumber ?? this.detachedRevision;
                await Promise.all(loadSummarizables);
            }
            else {
                await Promise.all([loadEditManager, ...loadSummarizables]);
            }
        }
        async loadSummarizable(summarizable, services) {
            return summarizable.load(scopeStorageService(services, summarizablesTreeKey, summarizable.key), (contents) => this.serializer.parse(contents));
        }
        /**
         * Submits an op to the Fluid runtime containing the given commit
         * @param commit - the commit to submit
         * @returns the submitted commit. This is undefined if the underlying `SharedObject` is not attached,
         * and may differ from `commit` due to enrichments like detached tree refreshers.
         */
        submitCommit(commit, schemaAndPolicy, isResubmit) {
            assert(this.isAttached() === (this.detachedRevision === undefined), 0x95a /* Detached revision should only be set when not attached */);
            const enrichedCommit = this.detachedRevision === undefined && !isResubmit
                ? this.commitEnricher.enrich(commit)
                : commit;
            // Edits submitted before the first attach are treated as sequenced because they will be included
            // in the attach summary that is uploaded to the service.
            // Until this attach workflow happens, this instance essentially behaves as a centralized data structure.
            if (this.detachedRevision !== undefined) {
                const newRevision = brand(this.detachedRevision + 1);
                this.detachedRevision = newRevision;
                this.editManager.addSequencedChange({ ...enrichedCommit, sessionId: this.editManager.localSessionId }, newRevision, this.detachedRevision);
                this.editManager.advanceMinimumSequenceNumber(newRevision, false);
                return undefined;
            }
            const message = this.messageCodec.encode({
                commit: enrichedCommit,
                sessionId: this.editManager.localSessionId,
            }, {
                idCompressor: this.idCompressor,
                schema: schemaAndPolicy,
            });
            this.submitLocalMessage(message, {
                // Clone the schema to ensure that during resubmit the schema has not been mutated by later changes
                schema: schemaAndPolicy.schema.clone(),
                policy: schemaAndPolicy.policy,
            });
            this.resubmitMachine.onCommitSubmitted(enrichedCommit);
        }
        processCore(message, local, localOpMetadata) {
            // Empty context object is passed in, as our decode function is schema-agnostic.
            const { commit, sessionId } = this.messageCodec.decode(message.contents, {
                idCompressor: this.idCompressor,
            });
            this.editManager.addSequencedChange({ ...commit, sessionId }, brand(message.sequenceNumber), brand(message.referenceSequenceNumber));
            this.resubmitMachine.onSequencedCommitApplied(local);
            this.editManager.advanceMinimumSequenceNumber(brand(message.minimumSequenceNumber));
        }
        getLocalBranch() {
            return this.editManager.localBranch;
        }
        onDisconnect() { }
        didAttach() {
            if (this.detachedRevision !== undefined) {
                this.detachedRevision = undefined;
            }
        }
        reSubmitCore(content, localOpMetadata) {
            // Empty context object is passed in, as our decode function is schema-agnostic.
            const { commit: { revision }, } = this.messageCodec.decode(this.serializer.decode(content), {
                idCompressor: this.idCompressor,
            });
            const [commit] = this.editManager.findLocalCommit(revision);
            // If a resubmit phase is not already in progress, then this must be the first commit of a new resubmit phase.
            if (this.resubmitMachine.isInResubmitPhase === false) {
                const toResubmit = this.editManager.getLocalCommits();
                assert(commit === toResubmit[0], 0x95d /* Resubmit phase should start with the oldest local commit */);
                this.resubmitMachine.prepareForResubmit(toResubmit);
            }
            assert(isClonableSchemaPolicy(localOpMetadata), 0x95e /* Local metadata must contain schema and policy. */);
            assert(this.resubmitMachine.isInResubmitPhase !== false, 0x984 /* Invalid resubmit outside of resubmit phase */);
            const enrichedCommit = this.resubmitMachine.peekNextCommit();
            this.submitCommit(enrichedCommit, localOpMetadata, true);
        }
        applyStashedOp(content) {
            // Empty context object is passed in, as our decode function is schema-agnostic.
            const { commit: { revision, change }, } = this.messageCodec.decode(content, { idCompressor: this.idCompressor });
            this.editManager.localBranch.apply({ change, revision });
        }
        getGCData(fullGC) {
            const gcNodes = super.getGCData(fullGC).gcNodes;
            for (const s of this.summarizables) {
                for (const [id, routes] of Object.entries(s.getGCData(fullGC).gcNodes)) {
                    gcNodes[id] ?? (gcNodes[id] = []);
                    for (const route of routes) {
                        // Non null asserting here because we are creating an array at gcNodes[id] if it is undefined
                        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
                        gcNodes[id].push(route);
                    }
                }
            }
            return {
                gcNodes,
            };
        }
    };
    __setFunctionName(_classThis, "SharedTreeCore");
    (() => {
        const _metadata = typeof Symbol === "function" && Symbol.metadata ? Object.create(_classSuper[Symbol.metadata] ?? null) : void 0;
        _summarizeCore_decorators = [throwIfBroken];
        __esDecorate(_classThis, null, _summarizeCore_decorators, { kind: "method", name: "summarizeCore", static: false, private: false, access: { has: obj => "summarizeCore" in obj, get: obj => obj.summarizeCore }, metadata: _metadata }, null, _instanceExtraInitializers);
        __esDecorate(null, _classDescriptor = { value: _classThis }, _classDecorators, { kind: "class", name: _classThis.name, metadata: _metadata }, null, _classExtraInitializers);
        SharedTreeCore = _classThis = _classDescriptor.value;
        if (_metadata) Object.defineProperty(_classThis, Symbol.metadata, { enumerable: true, configurable: true, writable: true, value: _metadata });
        __runInitializers(_classThis, _classExtraInitializers);
    })();
    return SharedTreeCore = _classThis;
})();
export { SharedTreeCore };
function isClonableSchemaPolicy(maybeSchemaPolicy) {
    const schemaAndPolicy = maybeSchemaPolicy;
    return schemaAndPolicy.schema !== undefined && schemaAndPolicy.policy !== undefined;
}
/**
 * Compose an {@link IChannelStorageService} which prefixes all paths before forwarding them to the original service
 */
function scopeStorageService(service, ...pathElements) {
    const scope = `${pathElements.join("/")}/`;
    return {
        async readBlob(path) {
            return service.readBlob(`${scope}${path}`);
        },
        async contains(path) {
            return service.contains(`${scope}${path}`);
        },
        async list(path) {
            return service.list(`${scope}${path}`);
        },
    };
}
//# sourceMappingURL=sharedTreeCore.js.map